unit FNCGridMain;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VCL.TMSFNCTypes, VCL.TMSFNCUtils, VCL.TMSFNCGraphics,
  VCL.TMSFNCGraphicsTypes, VCL.TMSFNCGridCell, VCL.TMSFNCGridOptions, VCL.TMSFNCCustomControl,
  VCL.TMSFNCCustomScrollControl, VCL.TMSFNCGridData, VCL.TMSFNCCustomGrid, VCL.TMSFNCGrid,
  Vcl.ExtCtrls, Data.DB, FireDAC.Stan.Intf, FireDAC.Stan.Option, FireDAC.Stan.Param,
  FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf, FireDAC.DApt.Intf, FireDAC.Stan.StorageBin,
  FireDAC.Comp.DataSet, FireDAC.Comp.Client, Vcl.StdCtrls;

const
	cstTicketColor = gcHoneyDew;

type
  TformMain = class(TForm)
    panelTicket: TPanel;
    gridTicket: TTMSFNCGrid;
    panelButtons: TPanel;
    dsOrderItems: TDataSource;
    tblOrderItems: TFDMemTable;
    tblOrderItemsForGridOnly: TFDMemTable;
    Memo1: TMemo;
    procedure gridTicketClick(Sender: TObject);
    procedure gridTicketMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure gridTicketSelectedCell(Sender: TObject; ACol, ARow: Integer);
    procedure FormCreate(Sender: TObject);
    procedure gridTicketGetCellLayout(Sender: TObject; ACol, ARow: Integer;
      ALayout: TTMSFNCGridCellLayout; ACellState: TTMSFNCGridCellState);
  private
	FTicketItemClicked : boolean;
	FChangeItem : boolean;
    procedure PrepareForm;
    procedure EditTicketItemEnter(nSelectedRow: integer);
    procedure SyncChangesToGrid(boPositionToEnd: boolean);
    procedure EditTicketItemExit;
    procedure AddAnItem;
    procedure EnableTicket(boValue: boolean);
  public
    { Public declarations }
  end;


var
  formMain: TformMain;

implementation

{$R *.dfm}

uses FNCGridHelper;


{ TformMain }

procedure TformMain.FormCreate(Sender: TObject);
begin
//	tblOrderItemsForGridOnly.LoadFromJSONFile('OrderItems.json');
	TCurrencyField(tblOrderItemsForGridOnly.FieldByName('Amount')).DisplayFormat := '#,##0.00;-#,##0.00; ';
	tblOrderItems.CloneCursor(tblOrderItemsForGridOnly, false, true);

	PrepareForm;
end;

procedure TformMain.gridTicketClick(Sender: TObject);
begin
	FTicketItemClicked := true;

	// The timing for settings the Selection.Mode is critical to have the
	// selection process work correctly.  It needs to be set in the Click
	// event, later (like the SelectedCell event) is too late.
	gridTicket.Options.Selection.Mode := TTMSFNCGridSelectionMode.smSingleRow;
end;

procedure TformMain.gridTicketGetCellLayout(Sender: TObject; ACol, ARow: Integer;
  ALayout: TTMSFNCGridCellLayout; ACellState: TTMSFNCGridCellState);
var
	sFieldName, sFieldValue : string;
	aField : TField;
	aDataSet : TDataSet;
	nGroupNumber : integer;
	nActiveRecord : integer;
begin
	if aRow = 0 then // title row
	begin
		if ((ACol = 0) or (ACol = 3)) then
			ALayout.Font.Size := 8      // make "Guest" smaller when used
		else
			ALayout.Font.Size := 10;
		ALayout.Font.Style := [];
		ALayout.TextAlign := TTMSFNCGraphicsTextAlign.gtaCenter;
		ALayout.Sides := [gsLeft, gsTop, gsRight, gsBottom];
		exit;
	end;

	aField := gridTicket.SyncDBegin(aCol, aRow, nActiveRecord);
	if aField <> nil then
	begin
		try
			if (aField.FieldName = 'Quantity') then
			begin
				ALayout.TextAlign := TTMSFNCGraphicsTextAlign.gtaCenter;
				if tblOrderItemsForGridOnly.FieldByName('Type').AsString = 'Z' then        // Make comments' quantities invisible
					ALayout.Font.Color := gridTicket.Color;
			end
			else
			if aField.FieldName = 'Amount' then
			begin
				ALayout.TextAlign := TTMSFNCGraphicsTextAlign.gtaTrailing;
				if (tblOrderItemsForGridOnly.FieldByName('Points Redeemed').AsInteger <> 0) then
					ALayout.Fill.Color := clTeal;
			end
			else
			if (aField.FieldName = 'Position at Table') then
			begin
				ALayout.TextAlign := TTMSFNCGraphicsTextAlign.gtaCenter;
				ALayout.Sides := [gsRight];
				ALayout.Font.Color := clBlue;
			end;

			if tblOrderItemsForGridOnly.FieldByName('Group Number').AsString <> '' then     // children items
			begin
				if tblOrderItemsForGridOnly.FieldByName('Group Number').AsInteger > 0 then	// Modifiers (ListButton, IncludeButton)
				begin
					if aField.FieldName = 'Quantity' then
						ALayout.Font.Color := cstTicketColor
					else
					if aField.FieldName = 'Receipt Description' then
					begin
						if Pos('  ', aField.AsString) = 1 then
						begin
							if Copy(aField.AsString, 3, 1) = '-' then
								ALayout.Font.Size := 8
							else
								ALayout.Font.Size := 10;
							ALayout.Font.Style := [];
						end
						else
						if Pos(' ', aField.AsString) = 1 then
						begin
							ALayout.Font.Size := 12;
							ALayout.Font.Style := [];
						end;
					end;
				end;
			end;

			if (aField.FieldName = 'Position at Table') then
			begin
				if (tblOrderItemsForGridOnly.FieldByName('Position At Table').AsInteger = 0) or
							(not tblOrderItemsForGridOnly.FieldByName('Group Leader').AsBoolean) then
					ALayout.Font.Color := gridTicket.Color;
			end;
		finally
			gridTicket.SyncDEnd(nActiveRecord);
		end;
	end;
end;

procedure TformMain.gridTicketMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
//	gridTicket.Options.Selection.Mode := TTMSFNCGridSelectionMode.smNone;
end;

procedure TformMain.gridTicketSelectedCell(Sender: TObject; ACol, ARow: Integer);
begin
	if FTicketItemClicked then
	begin
		FTicketItemClicked := false;

		tblOrderItems.Refresh;  // make sure tblOrderItems is aware of any changes done by others
		tblOrderItems.GoToCurrent(tblOrderItemsForGridOnly);

		EditTicketItemEnter(ARow);
	end;
end;

procedure TformMain.PrepareForm;
var
	aSettings : TGridSettings;
begin
	aSettings.Create(TGridType.gtBig);
	aSettings.DataSource := dsOrderItems;
	aSettings.AutoCreateColumns := false;
	aSettings.AllowColumnSorting := false;
	aSettings.UseAlternatingRowColors := false;
	aSettings.RowHeight := 24;
	aSettings.ShowVertBorders := false;
	aSettings.FontSize := 12;

	gridTicket.PrepareGrid(aSettings);
	gridTicket.AddColumn('Position at Table', 10, 'Guest');
	gridTicket.AddColumn('Quantity', 10, 'Qty');
	gridTicket.AddColumn('Receipt Description', 50, 'Description');
	gridTicket.AddColumn('Amount', 20, 'Amount');

	gridTicket.Color := gcHoneyDew;
//	gridTicket.Options.Selection.Mode := TTMSFNCGridSelectionMode.smNone;
	gridTicket.Active := true;
	gridTicket.ResizeColumns;
	gridTicket.ScrollMode := TTMSFNCCustomScrollControlMode.scmPixelScrolling;
end;


procedure TformMain.EditTicketItemEnter(nSelectedRow : integer);
var
	nCurrentProductGroup, nCurrentPrepLocation : integer;

	nCurrentGroupQuantity : double;
	nAmount : currency;
begin
	if FChangeItem then
    begin
		EditTicketItemExit;
		exit;
	end;

	if (tblOrderItems.RecordCount = 0) then
		exit;

	nCurrentProductGroup := tblOrderItems.FieldByName('Group Number').AsInteger;

	// Now check if the selected item was a group leader item
	if nCurrentProductGroup = 0 then
    begin
		// does product have the ability to have children
		if tblOrderItems.FieldByName('Group Leader').AsBoolean then
			nCurrentProductGroup := tblOrderItems.FieldByName('Number').AsInteger;
	end
    else
    begin
		if (nCurrentProductGroup <> tblOrderItems.FieldByName('Number').AsInteger) and (nSelectedRow > -1) then
		begin
			// this loop finds group leader/parent
			repeat
				tblOrderItems.Prior;
				Dec(nSelectedRow);
			until tblOrderItems.BOF or (nCurrentProductGroup = tblOrderItems.FieldByName('Number').AsInteger);

			if nSelectedRow > -1 then
				gridTicket.SelectRows(nSelectedRow, nSelectedRow);

			nCurrentProductGroup := tblOrderItems.FieldByName('Number').AsInteger;
		end;
	end;

	FChangeItem := true;

//	EnableTicket(false);

	AddAnItem;

//	EnableTicket(true);

	SyncChangesToGrid(false);
	FChangeItem := false;
end;

procedure TFormMain.AddAnItem;
begin
    // The ticket order items are edited or new items are added here using a different OrderItems table cursor
end;

procedure TformMain.EditTicketItemExit;
begin
	FChangeItem := false;
end;

procedure TformMain.SyncChangesToGrid(boPositionToEnd : boolean);
var
	nTopRow : integer;
begin
	if FChangeItem then
		nTopRow := gridTicket.TopRow;

	tblOrderItemsForGridOnly.Refresh; // This helps gridTicket display changes
	if FChangeItem then
		gridTicket.TopRow := nTopRow
	else
	begin
		if boPositionToEnd then
			tblOrderItemsForGridOnly.Last;  // AV error
	end;
end;

procedure TformMain.EnableTicket(boValue : boolean);
begin
	if boValue then
	begin
		gridTicket.Enabled := true;
//		gridTicket.Options.Selection.Mode := TTMSFNCGridSelectionMode.smNone;
	end
	else
	begin
		gridTicket.Enabled := false;
	end;
end;



end.
